@@ -1,7 +1,7 @@
 /*
  * alias.c - alias plugin for WeeChat: command aliases
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -36,7 +36,7 @@ WEECHAT_PLUGIN_DESCRIPTION(N_("Alias commands"));
 WEECHAT_PLUGIN_AUTHOR("Sébastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
-WEECHAT_PLUGIN_PRIORITY(8000);
+WEECHAT_PLUGIN_PRIORITY(ALIAS_PLUGIN_PRIORITY);
 
 #define ALIAS_IS_ARG_NUMBER(number) ((number >= '1') && (number <= '9'))
 
@@ -84,88 +84,50 @@ alias_search (const char *alias_name)
 {
     struct t_alias *ptr_alias;
 
+    if (!alias_name)
+        return NULL;
+
     for (ptr_alias = alias_list; ptr_alias;
          ptr_alias = ptr_alias->next_alias)
     {
-        if (weechat_strcasecmp (alias_name, ptr_alias->name) == 0)
+        if (strcmp (alias_name, ptr_alias->name) == 0)
             return ptr_alias;
     }
     return NULL;
 }
 
 /*
- * Adds a word to string and increments length.
+ * Adds word (in range) to alias string.
  */
 
 void
-alias_string_add_word (char **alias, int *length, const char *word)
-{
-    int length_word;
-    char *alias2;
-
-    if (!word)
-        return;
-
-    length_word = strlen (word);
-    if (length_word == 0)
-        return;
-
-    if (*alias == NULL)
-    {
-        *alias = malloc (length_word + 1);
-        strcpy (*alias, word);
-    }
-    else
-    {
-        alias2 = realloc (*alias, strlen (*alias) + length_word + 1);
-        if (!alias2)
-        {
-            if (*alias)
-            {
-                free (*alias);
-                *alias = NULL;
-            }
-            return;
-        }
-        *alias = alias2;
-        strcat (*alias, word);
-    }
-    *length += length_word;
-}
-
-/*
- * Adds word (in range) to string and increments length.
- */
-
-void
-alias_string_add_word_range (char **alias, int *length, const char *start,
-                             const char *end)
+alias_string_add_word_range (char **alias, const char *start, const char *end)
 {
     char *word;
 
     word = weechat_strndup (start, end - start);
     if (word)
     {
-        alias_string_add_word (alias, length, word);
+        weechat_string_dyn_concat (alias, word, -1);
         free (word);
     }
 }
 
 /*
- * Adds some arguments to string and increments length.
+ * Adds some arguments to alias string.
  */
 
 void
-alias_string_add_arguments (char **alias, int *length, char **argv, int start,
-                            int end)
+alias_string_add_arguments (char **alias, char **argv,
+                            int arg_start, int arg_end)
 {
     int i;
 
-    for (i = start; i <= end; i++)
+    for (i = arg_start; i <= arg_end; i++)
     {
-        if (i != start)
-            alias_string_add_word (alias, length, " ");
-        alias_string_add_word (alias, length, argv[i]);
+        if (i != arg_start)
+            weechat_string_dyn_concat (alias, " ", -1);
+        weechat_string_dyn_concat (alias, argv[i], -1);
     }
 }
 
@@ -178,22 +140,37 @@ alias_string_add_arguments (char **alias, int *length, char **argv, int start,
  *   $n-  arguments from n to last
  *   $n-m arguments from n to m
  *   $*   all arguments
+ *   $&   all arguments, with double quotes escaped (" replaced by \")
  *   $~   last argument
  *
+ * Example:
+ *
+ *   Parameters:
+ *     alias_args = "$2 $1 '$3-'"
+ *     user_args  = "abc def ghi jkl"
+ *
+ *   Result = "def abc 'ghi jkl'"
+ *
  * Note: result must be freed after use.
  */
 
 char *
 alias_replace_args (const char *alias_args, const char *user_args)
 {
-    char **argv, *res;
+    char **argv, **result, *temp;
     const char *start, *pos;
-    int n, m, argc, length_res, args_count, offset;
+    int n, m, argc, args_count, offset;
 
-    argv = weechat_string_split (user_args, " ", 0, 0, &argc);
+    if (!alias_args || !user_args)
+        return NULL;
 
-    res = NULL;
-    length_res = 0;
+    argv = weechat_string_split (user_args, " ", NULL,
+                                 WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                 | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                 | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                 0, &argc);
+
+    result = weechat_string_dyn_alloc (128);
     args_count = 0;
     start = alias_args;
     pos = start;
@@ -204,8 +181,8 @@ alias_replace_args (const char *alias_args, const char *user_args)
         if ((pos[0] == '\\') && (pos[1] == '$'))
         {
             offset = 2;
-            alias_string_add_word_range (&res, &length_res, start, pos);
-            alias_string_add_word (&res, &length_res, "$");
+            alias_string_add_word_range (result, start, pos);
+            weechat_string_dyn_concat (result, "$", -1);
         }
         else
         {
@@ -217,8 +194,22 @@ alias_replace_args (const char *alias_args, const char *user_args)
                     args_count++;
                     offset = 2;
                     if (pos > start)
-                        alias_string_add_word_range (&res, &length_res, start, pos);
-                    alias_string_add_word (&res, &length_res, user_args);
+                        alias_string_add_word_range (result, start, pos);
+                    weechat_string_dyn_concat (result, user_args, -1);
+                }
+                else if (pos[1] == '&')
+                {
+                    /* replace with all arguments, auto-escaping double quotes */
+                    args_count++;
+                    offset = 2;
+                    if (pos > start)
+                        alias_string_add_word_range (result, start, pos);
+                    temp = weechat_string_replace (user_args, "\"", "\\\"");
+                    if (temp)
+                    {
+                        weechat_string_dyn_concat (result, temp, -1);
+                        free (temp);
+                    }
                 }
                 else if (pos[1] == '~')
                 {
@@ -226,9 +217,9 @@ alias_replace_args (const char *alias_args, const char *user_args)
                     args_count++;
                     offset = 2;
                     if (pos > start)
-                        alias_string_add_word_range (&res, &length_res, start, pos);
+                        alias_string_add_word_range (result, start, pos);
                     if (argc > 0)
-                        alias_string_add_word (&res, &length_res, argv[argc - 1]);
+                        weechat_string_dyn_concat (result, argv[argc - 1], -1);
                 }
                 else if ((pos[1] == '-') && ALIAS_IS_ARG_NUMBER(pos[2]))
                 {
@@ -236,25 +227,25 @@ alias_replace_args (const char *alias_args, const char *user_args)
                     args_count++;
                     offset = 3;
                     if (pos > start)
-                        alias_string_add_word_range (&res, &length_res, start, pos);
+                        alias_string_add_word_range (result, start, pos);
                     if (pos[2] - '1' < argc)
                         m = pos[2] - '1';
                     else
                         m = argc - 1;
-                    alias_string_add_arguments (&res, &length_res, argv, 0, m);
+                    alias_string_add_arguments (result, argv, 0, m);
                 }
                 else if (ALIAS_IS_ARG_NUMBER(pos[1]))
                 {
                     args_count++;
                     n = pos[1] - '1';
                     if (pos > start)
-                        alias_string_add_word_range (&res, &length_res, start, pos);
+                        alias_string_add_word_range (result, start, pos);
                     if (pos[2] != '-')
                     {
                         /* replace with argument n */
                         offset = 2;
                         if (n < argc)
-                            alias_string_add_word (&res, &length_res, argv[n]);
+                            weechat_string_dyn_concat (result, argv[n], -1);
                     }
                     else
                     {
@@ -274,10 +265,7 @@ alias_replace_args (const char *alias_args, const char *user_args)
                             m = argc - 1;
                         }
                         if (n < argc)
-                        {
-                            alias_string_add_arguments (&res, &length_res,
-                                                        argv, n, m);
-                        }
+                            alias_string_add_arguments (result, argv, n, m);
                     }
                 }
             }
@@ -293,12 +281,12 @@ alias_replace_args (const char *alias_args, const char *user_args)
     }
 
     if (pos > start)
-        alias_string_add_word (&res, &length_res, start);
+        weechat_string_dyn_concat (result, start, -1);
 
     if (argv)
         weechat_string_free_split (argv);
 
-    return res;
+    return weechat_string_dyn_free (result, 0);
 }
 
 /*
@@ -312,7 +300,7 @@ alias_run_command (struct t_gui_buffer **buffer, const char *command)
     struct t_gui_buffer *old_current_buffer, *new_current_buffer;
 
     /* save current buffer pointer */
-    old_current_buffer = weechat_current_buffer();
+    old_current_buffer = weechat_current_buffer ();
 
     /* execute command */
     string = weechat_buffer_string_replace_local_var (*buffer, command);
@@ -322,7 +310,7 @@ alias_run_command (struct t_gui_buffer **buffer, const char *command)
         free (string);
 
     /* get new current buffer */
-    new_current_buffer = weechat_current_buffer();
+    new_current_buffer = weechat_current_buffer ();
 
     /*
      * if current buffer was changed by command, then we'll use this one for
@@ -337,18 +325,20 @@ alias_run_command (struct t_gui_buffer **buffer, const char *command)
  */
 
 int
-alias_cb (void *data, struct t_gui_buffer *buffer, int argc, char **argv,
+alias_cb (const void *pointer, void *data,
+          struct t_gui_buffer *buffer, int argc, char **argv,
           char **argv_eol)
 {
     struct t_alias *ptr_alias;
     char **commands, **ptr_cmd, **ptr_next_cmd;
-    char *args_replaced, *alias_command;
-    int some_args_replaced, length1, length2;
+    char *args_replaced, **alias_command;
+    int some_args_replaced;
 
     /* make C compiler happy */
+    (void) data;
     (void) argv;
 
-    ptr_alias = (struct t_alias *)data;
+    ptr_alias = (struct t_alias *)pointer;
 
     if (ptr_alias->running)
     {
@@ -382,26 +372,18 @@ alias_cb (void *data, struct t_gui_buffer *buffer, int argc, char **argv,
                  * arguments of the last command in the list (if no $1,$2,..$*)
                  * was found
                  */
-                if ((*ptr_next_cmd == NULL) && argv_eol[1] && (!some_args_replaced))
+                if ((*ptr_next_cmd == NULL) && argv_eol[1] && !some_args_replaced)
                 {
-                    length1 = strlen (*ptr_cmd);
-                    length2 = strlen (argv_eol[1]);
-
-                    alias_command = malloc (1 + length1 + 1 + length2 + 1);
+                    alias_command = weechat_string_dyn_alloc (128);
                     if (alias_command)
                     {
                         if (!weechat_string_is_command_char (*ptr_cmd))
-                            strcpy (alias_command, "/");
-                        else
-                            alias_command[0] = '\0';
-
-                        strcat (alias_command, *ptr_cmd);
-                        strcat (alias_command, " ");
-                        strcat (alias_command, argv_eol[1]);
-
-                        alias_run_command (&buffer,
-                                           alias_command);
-                        free (alias_command);
+                            weechat_string_dyn_concat (alias_command, "/", -1);
+                        weechat_string_dyn_concat (alias_command, *ptr_cmd, -1);
+                        weechat_string_dyn_concat (alias_command, " ", -1);
+                        weechat_string_dyn_concat (alias_command, argv_eol[1], -1);
+                        alias_run_command (&buffer, *alias_command);
+                        weechat_string_dyn_free (alias_command, 1);
                     }
                 }
                 else
@@ -413,14 +395,16 @@ alias_cb (void *data, struct t_gui_buffer *buffer, int argc, char **argv,
                     }
                     else
                     {
-                        alias_command = malloc (1 + strlen((args_replaced) ? args_replaced : *ptr_cmd) + 1);
+                        alias_command = weechat_string_dyn_alloc (128);
                         if (alias_command)
                         {
-                            strcpy (alias_command, "/");
-                            strcat (alias_command, (args_replaced) ? args_replaced : *ptr_cmd);
-                            alias_run_command (&buffer,
-                                               alias_command);
-                            free (alias_command);
+                            weechat_string_dyn_concat (alias_command, "/", -1);
+                            weechat_string_dyn_concat (
+                                alias_command,
+                                (args_replaced) ? args_replaced : *ptr_cmd,
+                                -1);
+                            alias_run_command (&buffer, *alias_command);
+                            weechat_string_dyn_free (alias_command, 1);
                         }
                     }
                 }
@@ -435,74 +419,6 @@ alias_cb (void *data, struct t_gui_buffer *buffer, int argc, char **argv,
     return WEECHAT_RC_OK;
 }
 
-/*
- * Frees an alias and remove it from list.
- */
-
-void
-alias_free (struct t_alias *alias)
-{
-    struct t_alias *new_alias_list;
-
-    /* remove alias from list */
-    if (last_alias == alias)
-        last_alias = alias->prev_alias;
-    if (alias->prev_alias)
-    {
-        (alias->prev_alias)->next_alias = alias->next_alias;
-        new_alias_list = alias_list;
-    }
-    else
-        new_alias_list = alias->next_alias;
-    if (alias->next_alias)
-        (alias->next_alias)->prev_alias = alias->prev_alias;
-
-    /* free data */
-    if (alias->hook)
-        weechat_unhook (alias->hook);
-    if (alias->name)
-        free (alias->name);
-    if (alias->command)
-        free (alias->command);
-    if (alias->completion)
-        free (alias->completion);
-    free (alias);
-
-    alias_list = new_alias_list;
-}
-
-/*
- * Frees all aliases.
- */
-
-void
-alias_free_all ()
-{
-    while (alias_list)
-    {
-        alias_free (alias_list);
-    }
-}
-
-/*
- * Searches for position of alias (to keep aliases sorted by name).
- */
-
-struct t_alias *
-alias_find_pos (const char *name)
-{
-    struct t_alias *ptr_alias;
-
-    for (ptr_alias = alias_list; ptr_alias; ptr_alias = ptr_alias->next_alias)
-    {
-        if (weechat_strcasecmp (name, ptr_alias->name) < 0)
-            return ptr_alias;
-    }
-
-    /* position not found (we will add to the end of list) */
-    return NULL;
-}
-
 /*
  * Hooks command for an alias.
  */
@@ -513,6 +429,12 @@ alias_hook_command (struct t_alias *alias)
     char *str_priority_name, *str_completion;
     int length;
 
+    if (alias->hook)
+    {
+        weechat_unhook (alias->hook);
+        alias->hook = NULL;
+    }
+
     /*
      * build string with priority and name: the alias priority is 2000, which
      * is higher than default one (1000), so the alias is executed before a
@@ -545,7 +467,7 @@ alias_hook_command (struct t_alias *alias)
                                         alias->command,
                                         NULL, NULL,
                                         (str_completion) ? str_completion : alias->completion,
-                                        &alias_cb, alias);
+                                        &alias_cb, alias, NULL);
 
     if (str_priority_name)
         free (str_priority_name);
@@ -553,6 +475,164 @@ alias_hook_command (struct t_alias *alias)
         free (str_completion);
 }
 
+/*
+ * Searches for position of alias (to keep aliases sorted by name).
+ */
+
+struct t_alias *
+alias_find_pos (const char *name)
+{
+    struct t_alias *ptr_alias;
+
+    for (ptr_alias = alias_list; ptr_alias; ptr_alias = ptr_alias->next_alias)
+    {
+        if (weechat_strcmp (name, ptr_alias->name) < 0)
+            return ptr_alias;
+    }
+
+    /* position not found (we will add to the end of list) */
+    return NULL;
+}
+
+/*
+ * Inserts alias in list of aliases.
+ */
+
+void
+alias_insert (struct t_alias *alias)
+{
+    struct t_alias *pos_alias;
+
+    if (alias_list)
+    {
+        pos_alias = alias_find_pos (alias->name);
+        if (pos_alias)
+        {
+            /* insert alias into the list (before alias found) */
+            alias->prev_alias = pos_alias->prev_alias;
+            alias->next_alias = pos_alias;
+            if (pos_alias->prev_alias)
+                (pos_alias->prev_alias)->next_alias = alias;
+            else
+                alias_list = alias;
+            pos_alias->prev_alias = alias;
+        }
+        else
+        {
+            /* add alias to end of list */
+            alias->prev_alias = last_alias;
+            alias->next_alias = NULL;
+            last_alias->next_alias = alias;
+            last_alias = alias;
+        }
+    }
+    else
+    {
+        alias->prev_alias = NULL;
+        alias->next_alias = NULL;
+        alias_list = alias;
+        last_alias = alias;
+    }
+}
+
+/*
+ * Removes alias from list of aliases.
+ */
+
+void
+alias_remove_from_list (struct t_alias *alias)
+{
+    if (last_alias == alias)
+        last_alias = alias->prev_alias;
+    if (alias->prev_alias)
+        (alias->prev_alias)->next_alias = alias->next_alias;
+    else
+        alias_list = alias->next_alias;
+    if (alias->next_alias)
+        (alias->next_alias)->prev_alias = alias->prev_alias;
+}
+
+/*
+ * Renames an alias.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+alias_rename (struct t_alias *alias, const char *new_name)
+{
+    struct t_config_option *ptr_option;
+
+    if (!alias || !new_name || !new_name[0] || alias_search (new_name))
+        return 0;
+
+    /* rename options */
+    ptr_option = weechat_config_search_option (
+        alias_config_file,
+        alias_config_section_cmd,
+        alias->name);
+    if (ptr_option)
+        weechat_config_option_rename (ptr_option, new_name);
+    ptr_option = weechat_config_search_option (
+        alias_config_file,
+        alias_config_section_completion,
+        alias->name);
+    if (ptr_option)
+        weechat_config_option_rename (ptr_option, new_name);
+
+    /* rename alias */
+    free (alias->name);
+    alias->name = strdup (new_name);
+
+    /* hook command again */
+    alias_hook_command (alias);
+
+    /* move alias in list (to keep list sorted) */
+    alias_remove_from_list (alias);
+    alias_insert (alias);
+
+    return 1;
+}
+
+/*
+ * Frees an alias and remove it from list.
+ */
+
+void
+alias_free (struct t_alias *alias)
+{
+    if (!alias)
+        return;
+
+    alias_remove_from_list (alias);
+
+    /* free data */
+    if (alias->hook)
+        weechat_unhook (alias->hook);
+    if (alias->name)
+        free (alias->name);
+    if (alias->command)
+        free (alias->command);
+    if (alias->completion)
+        free (alias->completion);
+    free (alias);
+}
+
+/*
+ * Frees all aliases.
+ */
+
+void
+alias_free_all ()
+{
+    while (alias_list)
+    {
+        alias_free (alias_list);
+    }
+}
+
 /*
  * Updates completion for an alias.
  */
@@ -565,12 +645,37 @@ alias_update_completion (struct t_alias *alias, const char *completion)
         free (alias->completion);
     alias->completion = (completion) ? strdup (completion) : NULL;
 
-    /* unhook and hook again command, with new completion */
-    weechat_unhook (alias->hook);
-    alias->hook = NULL;
+    /* hook command again, with new completion */
     alias_hook_command (alias);
 }
 
+/*
+ * Checks if an alias name is valid: it must not contain any slashes nor
+ * any spaces.
+ *
+ * Returns:
+ *   1: name is valid
+ *   0: name is invalid
+ */
+
+int
+alias_name_valid (const char *name)
+{
+    if (!name || !name[0])
+        return 0;
+
+    /* no spaces allowed */
+    if (strchr (name, ' '))
+        return 0;
+
+    /* no slashes allowed */
+    if (strchr (name, '/'))
+        return 0;
+
+    /* name is valid */
+    return 1;
+}
+
 /*
  * Creates a new alias and adds it to alias list.
  *
@@ -580,9 +685,18 @@ alias_update_completion (struct t_alias *alias, const char *completion)
 struct t_alias *
 alias_new (const char *name, const char *command, const char *completion)
 {
-    struct t_alias *new_alias, *ptr_alias, *pos_alias;
+    struct t_alias *new_alias, *ptr_alias;
 
-    if (!name || !name[0] || !command || !command[0])
+    if (!alias_name_valid (name))
+    {
+        weechat_printf (NULL,
+                        _("%s%s: invalid alias name: \"%s\""),
+                        weechat_prefix ("error"), ALIAS_PLUGIN_NAME,
+                        name);
+        return NULL;
+    }
+
+    if (!command || !command[0])
         return NULL;
 
     while (weechat_string_is_command_char (name))
@@ -605,36 +719,7 @@ alias_new (const char *name, const char *command, const char *completion)
 
         alias_hook_command (new_alias);
 
-        if (alias_list)
-        {
-            pos_alias = alias_find_pos (name);
-            if (pos_alias)
-            {
-                /* insert alias into the list (before alias found) */
-                new_alias->prev_alias = pos_alias->prev_alias;
-                new_alias->next_alias = pos_alias;
-                if (pos_alias->prev_alias)
-                    (pos_alias->prev_alias)->next_alias = new_alias;
-                else
-                    alias_list = new_alias;
-                pos_alias->prev_alias = new_alias;
-            }
-            else
-            {
-                /* add alias to end of list */
-                new_alias->prev_alias = last_alias;
-                new_alias->next_alias = NULL;
-                last_alias->next_alias = new_alias;
-                last_alias = new_alias;
-            }
-        }
-        else
-        {
-            new_alias->prev_alias = NULL;
-            new_alias->next_alias = NULL;
-            alias_list = new_alias;
-            last_alias = new_alias;
-        }
+        alias_insert (new_alias);
     }
 
     return new_alias;
