@@ -1,7 +1,7 @@
 /*
  * relay-server.c - server functions for relay plugin
  *
- * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -51,17 +51,17 @@ struct t_relay_server *last_relay_server = NULL;
  *
  * Examples:
  *
- *   string                    ipv4 ipv6 ssl unix protocol protocol_args
+ *   string                    ipv4 ipv6 tls unix protocol protocol_args
  *   ---------------------------------------------------------------
  *   irc.libera                1    1    0   0    irc      libera
- *   ssl.irc.libera            1    1    1   0    irc      libera
+ *   tls.irc.libera            1    1    1   0    irc      libera
  *   ipv4.irc.libera           1    0    0   0    irc      libera
  *   ipv6.irc.libera           0    1    0   0    irc      libera
  *   ipv4.ipv6.irc.libera      1    1    0   0    irc      libera
- *   ipv6.ssl.irc.libera       0    1    1   0    irc      libera
+ *   ipv6.tls.irc.libera       0    1    1   0    irc      libera
  *   weechat                   1    1    0   0    weechat
- *   ssl.weechat               1    1    1   0    weechat
- *   ipv6.ssl.weechat          0    1    1   0    weechat
+ *   tls.weechat               1    1    1   0    weechat
+ *   ipv6.tls.weechat          0    1    1   0    weechat
  *   unix.weechat              0    0    0   1    weechat
  *
  * Note: *protocol and *protocol_args must be freed after use.
@@ -69,16 +69,16 @@ struct t_relay_server *last_relay_server = NULL;
 
 void
 relay_server_get_protocol_args (const char *protocol_and_args,
-                                int *ipv4, int *ipv6, int *ssl,
+                                int *ipv4, int *ipv6, int *tls,
                                 int *unix_socket,
                                 char **protocol, char **protocol_args)
 {
-    int opt_ipv4, opt_ipv6, opt_ssl, opt_unix_socket;
+    int opt_ipv4, opt_ipv6, opt_tls, opt_unix_socket;
     char *pos;
 
     opt_ipv4 = -1;
     opt_ipv6 = -1;
-    opt_ssl = 0;
+    opt_tls = 0;
     opt_unix_socket = -1;
     while (1)
     {
@@ -92,9 +92,9 @@ relay_server_get_protocol_args (const char *protocol_and_args,
             opt_ipv6 = 1;
             protocol_and_args += 5;
         }
-        else if (strncmp (protocol_and_args, "ssl.", 4) == 0)
+        else if (strncmp (protocol_and_args, "tls.", 4) == 0)
         {
-            opt_ssl = 1;
+            opt_tls = 1;
             protocol_and_args += 4;
         }
         else if (strncmp (protocol_and_args, "unix.", 5) == 0)
@@ -134,8 +134,8 @@ relay_server_get_protocol_args (const char *protocol_and_args,
         *ipv4 = opt_ipv4;
     if (ipv6)
         *ipv6 = opt_ipv6;
-    if (ssl)
-        *ssl = opt_ssl;
+    if (tls)
+        *tls = opt_tls;
     if (unix_socket)
         *unix_socket = opt_unix_socket;
 
@@ -170,6 +170,9 @@ relay_server_search (const char *protocol_and_args)
 {
     struct t_relay_server *ptr_server;
 
+    if (!protocol_and_args)
+        return NULL;
+
     for (ptr_server = relay_servers; ptr_server;
          ptr_server = ptr_server->next_server)
     {
@@ -213,6 +216,9 @@ relay_server_search_path (const char *path)
 {
     struct t_relay_server *ptr_server;
 
+    if (!path)
+        return NULL;
+
     for (ptr_server = relay_servers; ptr_server;
          ptr_server = ptr_server->next_server)
     {
@@ -276,7 +282,6 @@ relay_server_sock_cb (const void *pointer, void *data, int fd)
     (void) data;
     (void) fd;
 
-    client_fd = -1;
     relay_password = NULL;
     relay_totp_secret = NULL;
 
@@ -507,7 +512,7 @@ relay_server_create_socket (struct t_relay_server *server)
         server_addr6.sin6_addr = in6addr_any;
         if (bind_address && bind_address[0])
         {
-            if (!inet_pton (domain, bind_address, &server_addr6.sin6_addr))
+            if (inet_pton (domain, bind_address, &server_addr6.sin6_addr) != 1)
             {
                 weechat_printf (NULL,
                                 /* TRANSLATORS: second "%s" is "IPv4" or "IPv6" */
@@ -529,7 +534,7 @@ relay_server_create_socket (struct t_relay_server *server)
         server_addr.sin_addr.s_addr = INADDR_ANY;
         if (bind_address && bind_address[0])
         {
-            if (!inet_pton (domain, bind_address, &server_addr.sin_addr))
+            if (inet_pton (domain, bind_address, &server_addr.sin_addr) != 1)
             {
                 weechat_printf (NULL,
                                 /* TRANSLATORS: second "%s" is "IPv4" or "IPv6" */
@@ -788,7 +793,7 @@ relay_server_create_socket (struct t_relay_server *server)
 struct t_relay_server *
 relay_server_new (const char *protocol_string, enum t_relay_protocol protocol,
                   const char *protocol_args, int port, const char *path,
-                  int ipv4, int ipv6, int ssl, int unix_socket)
+                  int ipv4, int ipv6, int tls, int unix_socket)
 {
     struct t_relay_server *new_server, *dup_server;
     struct t_hashtable *options;
@@ -837,7 +842,7 @@ relay_server_new (const char *protocol_string, enum t_relay_protocol protocol,
             weechat_hashtable_free (options);
         new_server->ipv4 = ipv4;
         new_server->ipv6 = ipv6;
-        new_server->ssl = ssl;
+        new_server->tls = tls;
         new_server->unix_socket = unix_socket;
         new_server->sock = -1;
         new_server->hook_fd = NULL;
@@ -1005,7 +1010,7 @@ relay_server_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "ipv6", server->ipv6))
         return 0;
-    if (!weechat_infolist_new_var_integer (ptr_item, "ssl", server->ssl))
+    if (!weechat_infolist_new_var_integer (ptr_item, "tls", server->tls))
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "unix_socket", server->unix_socket))
         return 0;
@@ -1044,7 +1049,7 @@ relay_server_print_log ()
         weechat_log_printf ("  path. . . . . . . . . : %s",    ptr_server->path);
         weechat_log_printf ("  ipv4. . . . . . . . . : %d",    ptr_server->ipv4);
         weechat_log_printf ("  ipv6. . . . . . . . . : %d",    ptr_server->ipv6);
-        weechat_log_printf ("  ssl . . . . . . . . . : %d",    ptr_server->ssl);
+        weechat_log_printf ("  tls . . . . . . . . . : %d",    ptr_server->tls);
         weechat_log_printf ("  unix_socket . . . . . : %d",    ptr_server->unix_socket);
         weechat_log_printf ("  sock. . . . . . . . . : %d",    ptr_server->sock);
         weechat_log_printf ("  hook_fd . . . . . . . : 0x%lx", ptr_server->hook_fd);
